#
# spec file for package rr
#
# Copyright (c) 2025 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


Name:           rr
Version:        5.9.0
Release:        1.3
Summary:        Records nondeterministic executions and debugs them deterministically
License:        MIT
Group:          Development/Languages/C and C++
URL:            https://rr-project.org/
Source:         https://github.com/mozilla/%{name}/archive/%{version}.tar.gz
BuildRequires:  capnproto
BuildRequires:  cmake >= 3.5.0
BuildRequires:  gcc-c++
BuildRequires:  gdb
BuildRequires:  libcapnp-devel
BuildRequires:  libzstd-devel
BuildRequires:  patchelf
BuildRequires:  pkgconfig
BuildRequires:  python3-base
BuildRequires:  python3-pexpect
BuildRequires:  pkgconfig(expat)
BuildRequires:  pkgconfig(zlib)
ExclusiveArch:  x86_64 aarch64
%ifarch x86_64
BuildRequires:  gcc-c++-32bit
%endif

%package zsh-completion
Summary:        Zsh completion for %{name}
Group:          Development/Tools/Version Control
Requires:       %{name} = %{version}
Requires:       zsh
Supplements:    (%{name} and zsh)
BuildArch:      noarch

%description zsh-completion
Zsh command line completion support for %{name}.

%description
This program aspires to be your primary debugging tool, enhancing gdb. It
also provides efficient reverse execution under gdb. Set breakpoints and
data watchpoints and quickly reverse-execute to where they were hit.

%prep
%autosetup -p1

%build
# Fix incorrect path to bash
sed -i "s|%{_bindir}/bash|/bin/bash|g" ./scripts/signal-rr-recording.sh
%cmake \
  -DBUILD_TESTS=OFF \
%ifarch aarch64
  -Ddisable32bit=true \
%endif
  -DCMAKE_BUILD_TYPE=Release \
  -DCMAKE_CXX_STANDARD=17
%cmake_build

%install
%cmake_install
%python3_fix_shebang

%files
%license LICENSE
%doc README.md
%dir %{_datadir}/rr
%dir %{_libdir}/rr
%{_bindir}/rr
%{_bindir}/rr_exec_stub*
%{_bindir}/signal-rr-recording.sh
%{_bindir}/rr-collect-symbols.py
%{_libdir}/rr/librrpreload*.so
%ifarch x86_64
%{_libdir}/rr/librraudit_32.so
%{_libdir}/rr/librrpage_32.so
%endif
%{_libdir}/rr/librraudit.so
%{_libdir}/rr/librrpage.so
%{_datadir}/rr/*
%{_datadir}/bash-completion/completions/rr

%files zsh-completion
%{_datadir}/zsh

%changelog
* Fri Feb 14 2025 Michael Vetter <mvetter@suse.com>
- Update to 5.9.0:
  * As usual there are lots of small improvements and bugfixes.
    The main feature of this release is that in kernels >= 6.10,
    rr now works with perf_event_paranoid set to 2, which is the
    default for most distributions. This requires a kernel bug fix
    by Kyle Huey that only shipped in 6.10. rr is slightly more
    efficient with perf_event_paranoid set to 1, but that won't
    matter in practice.
* Tue May 21 2024 Michael Vetter <mvetter@suse.com>
- Update to 5.8.0:
  * This release contains many small bug fixes and updates for new
    syscalls and CPU microarchitectures.
  * There is some support for lldb now. rr replay -d lldb will
    launch LLDB and connect it to rr just like we do for gdb.
    LLDB does not expose reverse-execution commands so for now,
    you can only replay forwards. (Of course you can still replay a
    recording as many times as you need to.)
* Tue Mar 12 2024 pgajdos@suse.com
- remove dependency on /usr/bin/python3 using
  %%python3_fix_shebang macro, [bsc#1212476]
* Wed Oct  4 2023 Michael Vetter <mvetter@suse.com>
- Update to 5.7.0:
  * This release includes many bug fixes, improvements to system
    call coverage, and incremental performance improvements.
    In particular, performance is improved for applications with
    thousands of threads (partly via the new WaitManager).
    A lot of rare intermittent bugs have been fixed, partly due
    to significant reworking of how unexpected task exits are handled.
- Remove upstream patches:
  * 2248c85c.patch
  * 2979c60e.patch
* Wed May 24 2023 Michael Vetter <mvetter@suse.com>
- Drop e32502d0.patch:
  This was added due to a build fail related to:
  https://github.com/rr-debugger/rr/issues/3485
  Which seems to have been a bug in gcc:
  https://gcc.gnu.org/bugzilla/show_bug.cgi?id=109420
  But now it seems that it fails our build and without this patch
  it builds fine
* Wed Apr 12 2023 Michael Vetter <mvetter@suse.com>
- Add e32502d0.patch: Avoid having type members and enum members
  of Arch with the same name
* Thu Mar 30 2023 Michael Vetter <mvetter@suse.com>
- Add 2248c85c.patch: Fix build on Tumbleweed with gcc13 by
  explicitly including cstdint
* Mon Oct 31 2022 Dominique Leuenberger <dimstar@opensuse.org>
- Add 2979c60e.patch: Avoid creating a struct with elements after
  ethtool_sset_info's variable-length-array.
* Tue Aug 16 2022 Dirk Müller <dmueller@suse.com>
- filelist fix for aarch64
* Mon Aug  8 2022 Michael Vetter <mvetter@suse.com>
- Update to 5.6.0:
  * This release includes many bug fixes, improvements to system call coverage, and
    incremental performance improvements. Plus:
  * ARM Aarch64 support is now production quality (features implemented and
    tests pass), with some caveats. Recent iterations of Cortex and Neoverse (AWS
    Graviton) chips are supported, as is Apple M1 (when running Linux on bare metal
    M1). LDREX/STREX are not supported; userspace code must use Large System
    Extensions atomics only. Thanks to Yichao Yu and Keno Fischer for this work! We
    don't have prebuilt ARM packages yet.
  * Recording applications that use RDTSC heavily should be much faster than in
    previous versions of rr.
  * The rseq system call is now supported (via emulation).
-  Remove patches contained in new release:
  * rr-glib-234.patch
  * rr-map-elf-header.patch
  * rr-pthread-for-new-gcc.patch
* Mon May 23 2022 Michael Vetter <mvetter@suse.com>
- Add rr-glib-234.patch to build with glib > 2.34
  See https://github.com/rr-debugger/rr/issues/3199
* Fri Apr 29 2022 Michael Vetter <mvetter@suse.com>
- Cmake changes:
  * build as release
  * dont build 32bit
  * set c++14 via cmake instead of exporting variable
  * use cmake_build macro
- Add rr-map-elf-header.patch:
  Fix undefined reference to `_dl_addr'
- Add rr-pthread-for-new-gcc.patch:
  Fix https://github.com/rr-debugger/rr/issues/2987
* Wed Dec  1 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Enable build on aarch64
* Mon Sep 20 2021 Michael Vetter <mvetter@suse.com>
- Update to 5.5.0:
  * As usual there are many bug fixes, improvements to system call
    coverage, and incremental performance improvements.
    See: https://github.com/rr-debugger/rr/compare/5.4.0...5.5.0
* Sun Nov 29 2020 Bernhard Wiedemann <bwiedemann@suse.com>
- Only build for x86_64
* Thu Oct 29 2020 Michael Vetter <mvetter@suse.com>
- Update to 5.4.0:
  * As usual there are many bug fixes, improvements to system call
    coverage, and incremental performance improvements.
  * AMD Zen CPUs are now supported, with some caveats.
  * There is experimental support for ARM AArch64 platforms that
    avoid the STREX instruction.
  * rr record has a new option --nested=detach which lets nested
    rr recordings run "detached" to create a separate recording.
* Wed Dec 18 2019 Michael Vetter <mvetter@suse.com>
- Update to 5.3.0:
  * This is primarily a maintenance release.
    There are many bug fixes, improvements to system call coverage,
  and incremental performance improvements.
  * The rr sources, rr buildid and rr traceinfo commands have been
    added to make it easier for external tools to work with rr
  traces. rr ls has been added to improve management of traces.
- Remove:
  * rr-5.2.0-cpp14.patch
  * rr-5.2.0-python3.patch
  * rr-5.2.0-remove-ucontext.patch
* Tue Dec 17 2019 Michael Vetter <mvetter@suse.com>
- Add rr-5.2.0-python3.patch
  We remove Python2 soon.
  Backport Python3 changes planned for next release only.
  See https://github.com/mozilla/rr/issues/2390
* Tue Dec 17 2019 Michael Vetter <mvetter@suse.com>
- Add patches to fix build:
  * rr-5.2.0-cpp14.patch
    See https://github.com/mozilla/rr/issues/2391
  * rr-5.2.0-remove-ucontext.patch
    See https://github.com/mozilla/rr/issues/2269
* Wed Oct  9 2019 mvetter@suse.com
- Update to 5.2.0:
  * Improvements to chaos mode.
  * Improvements to trace portability:
    https://robert.ocallahan.org/2018/04/cpuid-features-xsave-and-rr-trace.html
  * Various bug fixes.
* Thu Dec 14 2017 mvetter@suse.com
- Update to 5.1.0:
  * gdb info proc map supported
  * Maintenance update to handle issues arising from Ubuntu 17.10 and Fedora 27
  * Various bugs fixed
- Remove rr-shared.patch: contained in upstream
* Mon Nov 27 2017 mpluskal@suse.com
- Cleanup with spec-cleaner
- Use more cmake macros:
  * rr-shared.patch
  * Disable building of tests as it breaks with our optflags
- Fix dependency on /usr/bin/bash
- Build also 32-bit support
- Limit to building on supported platforms
* Wed Oct 25 2017 mvetter@suse.com
- Update to 5.0.0:
  Introduction of Cap'n Proto to stabilize the recording format.
  Recordings created in this rr release should be replayable in any
  future rr release.
  New rr pack command makes recordings self-contained.
  Recordings can be made on one machine and replayed on a different
  machine by trapping CPUID instructions when supported on the replay
  machine.
  Brotli compression for smaller traces and lower recording overhead.
  rr replay command line argument -- can be followed by any
  number of arguments to be passed to the debugger.
  rr now chooses replay as the default subcommand if
  the first argument is a directory.
  Performance improvements.
  Syscall support expanded.
  Many bugs fixed.
- Remove: assert_typos.patch and major-minor-macro.patch
* Thu Apr 13 2017 mvetter@suse.com
- Use regular make_install macro. No build directory here.
* Fri Mar 17 2017 mvetter@suse.com
- Add assert_typos.patch:
  Fixes some typos in the tests.
* Fri Mar 17 2017 mvetter@suse.com
- Add major-minor-macro.patch:
  With new glibc version we need to include sys/sysmacros.h
  explicitly.
* Tue Mar  7 2017 mvetter@suse.com
- Initial version of 4.5.0 for openSUSE
